// toShare reservations.js
'use strict';
// uses rest, tooltip

const sectionReservations = document.querySelector('.reservations');
const buttonNew = sectionReservations.querySelector('button.new');
const buttonUp = sectionReservations.querySelector('button.up');
const buttonDown = sectionReservations.querySelector('button.down');
const tableReservation = sectionReservations.querySelector('table');
const tbodyReservation = sectionReservations.querySelector('tbody');
const trTemplateReservation = tbodyReservation.firstElementChild.cloneNode(true);
const divEditLayer = document.querySelector('div.edit-layer');
const buttonCancel = divEditLayer.querySelector('button.cancel');
const inputTableId = divEditLayer.querySelector('input.table-id');
const inputTableName = divEditLayer.querySelector('input.table-name');
const selectTableUnit = divEditLayer.querySelector('select.table-unit');
const inputTableUri = divEditLayer.querySelector('input.table-custom-uri');
const inputTableDesc = divEditLayer.querySelector('input.table-desc');
const inputItemName = divEditLayer.querySelector('input.item-name');
const checkboxItemBulk = divEditLayer.querySelector('input.item-bulk');
const checkboxDisable = divEditLayer.querySelector('input.disable');
const labelDisable = divEditLayer.querySelector('label.disable');
const buttonAdd = divEditLayer.querySelector('button.add');
const buttonFix = divEditLayer.querySelector('button.fix');
const buttonDel = divEditLayer.querySelector('button.del');
const UNSELECT = -1;
let orderStartIndex = UNSELECT;
let orderEndIndex = UNSELECT;

async function updateReservationList(){
	await atsReservationTables.restGet(TABLE_ID_ALL, WITHDISABLED);
	tbodyReservation.innerHTML = '';
	for(const atsTable of atsReservationTables.data){
		const aTag = document.createElement('a');
		aTag.href = 'items';
		aTag.innerText = '(編集)'  + atsTable.item_name;
		aTag.hash = atsTable.table_id;
		const trTable = trTemplateReservation.cloneNode(true);
		trTable.dataset.tableId = atsTable.table_id;
		if(atsTable.disable){trTable.classList.add('disable');}
		const td = trTable.querySelectorAll('td');
		td[0].innerText = atsTable.table_name;
		td[1].innerText = (atsTable.table_custom) ?  atsTable.table_uri: selectTableUnit.namedItem(atsTable.table_uri).label;
		td[2].innerText = atsTable.table_desc;
		td[3].textContent = '';
		td[3].appendChild(aTag);
		td[4].textContent = (atsTable.item_bulk)? '可' : '';
		td[5].textContent = (atsTable.disable)? '非表示' : '';

		td[0].addEventListener('click', onTdReservationClick);
		td[1].addEventListener('click', onTdReservationClick);
		td[2].addEventListener('click', onTdReservationClick);
		td[4].addEventListener('click', onTdReservationClick);
		td[5].addEventListener('click', onTdReservationClick);
		td[6].addEventListener('click', onTdOrderClick);
		tbodyReservation.appendChild(trTable);
	}
}

function redrawSelected(){
	buttonUp.disabled = (orderStartIndex===UNSELECT)||(orderStartIndex===0);
	buttonDown.disabled = (orderEndIndex===UNSELECT)||(orderEndIndex===tbodyReservation.rows.length-1);
	for(const tbodyTr of tbodyReservation.querySelectorAll('tr')){
		if(tbodyTr.classList.contains('select')){tbodyTr.classList.remove('select');}
	}
	if(orderStartIndex != UNSELECT){
		for(let index=orderStartIndex; index <= orderEndIndex; index++){
			tbodyReservation.rows[index].classList.add('select');
		}
	}
}

function onResize(){
	const remPx = parseFloat(getComputedStyle(document.documentElement).fontSize);
	const viewHeight = window.innerHeight;
	const conteinerTop = sectionReservations.getBoundingClientRect().top;
	const containerHeight = viewHeight - conteinerTop - 6 * remPx;
	tableReservation.style.maxHeight = containerHeight + 'px';
}
function onNewEdit(){
	for(const tbodyTr of tbodyReservation.querySelectorAll('tr')){
		if(tbodyTr.classList.contains('current')){tbodyTr.classList.remove('current');}
	}
	divEditLayer.style.display = 'block';
	inputTableId.value = '0';
	inputTableName.value = '';
	selectTableUnit.slectedIndex = 1;
	inputTableUri.value = '';
	inputTableUri.style.visibility = 'hidden';
	inputTableDesc.value = '';
	inputItemName.value = 'アイテム';
	checkboxItemBulk.checked = false;
	inputTableName.focus();
	checkboxDisable.style.visibility = 'hidden';
	labelDisable.style.visibility = 'hidden';
	buttonAdd.disabled = true;
	buttonFix.disabled = true;
	buttonDel.disabled = true;
}

async function onOrderExecute(){
	let tableIds = [];
	for(let index =orderStartIndex; index <= orderEndIndex; index++){
		tableIds.push(atsReservationTables.data[index].table_id);
	}
	switch(event.target){
		case buttonUp:{
			tableIds.push(atsReservationTables.data[orderStartIndex-1].table_id);
			try{
				await atsReservationTables.restOrderPut(tableIds)
				orderStartIndex -=1;
				orderEndIndex -=1;
			}catch(e){confirm(e.message);}
			break;
		}
		case buttonDown:{
			tableIds.unshift(atsReservationTables.data[orderEndIndex+1].table_id);
			try{
				await atsReservationTables.restOrderPut(tableIds)
				orderStartIndex +=1;
				orderEndIndex +=1;
			}catch(e){confirm(e.message);}
			break;
		}
		default: confirm('onExecute tagert unknown');
	}
	await updateReservationList();
	redrawSelected();
}

function onTdReservationClick(event){
	orderStartIndex = UNSELECT;
	orderEndIndex = UNSELECT;
	redrawSelected();
	if((event.currentTarget.parentElement.tagName != 'A')&(!event.currentTarget.parentElement.classList.contains('current'))){
		for(const tbodyTr of tbodyReservation.querySelectorAll('tr')){
			if(tbodyTr.classList.contains('current')){tbodyTr.classList.remove('current');}
		}
		event.currentTarget.parentElement.classList.add('current');
		divEditLayer.style.display = 'block';
		event.preventDefault();
		const atsTable = atsReservationTables.findField('table_id', Number(event.currentTarget.parentElement.dataset.tableId));
		inputTableId.value = atsTable.table_id;
		inputTableName.value = atsTable.table_name;
		inputTableName.select();
		inputTableName.focus();
		selectTableUnit.selectedIndex = 1;
		if(atsTable.table_custom){
			selectTableUnit.selectedIndex = selectTableUnit.namedItem('custom').index;
			inputTableUri.value = atsTable.table_uri;
			inputTableUri.style.visibility = 'visible';
		}else{
			selectTableUnit.selectedIndex = selectTableUnit.namedItem(atsTable.table_uri).index;
			inputTableUri.value = '';
			inputTableUri.style.visibility = 'hidden';
		}
		inputTableDesc.value = atsTable.table_desc;
		inputItemName.value = atsTable.item_name;
		checkboxItemBulk.checked = (atsTable.item_bulk===1);
		checkboxDisable.style.visibility = 'visible';
		labelDisable.style.visibility = 'visible';
		checkboxDisable.checked = (atsTable.disable===1);
		buttonAdd.disabled = true;
		buttonFix.disabled = true;
		buttonDel.disabled = false;
	}else{
		event.currentTarget.parentElement.classList.remove('current');
		divEditLayer.style.display = 'none';
	}
}

function onTdOrderClick(event){
	let index = event.currentTarget.parentElement.rowIndex -1;
	if((orderStartIndex > index)||(orderStartIndex===UNSELECT)){
		orderStartIndex = index;
		orderEndIndex = index;
	}else if(orderStartIndex===index){
		orderStartIndex = UNSELECT;
		orderEndIndex = UNSELECT;
	}else if(orderStartIndex < index){
		orderEndIndex = index;
	};
	redrawSelected();
}

function onOutsideClick(event){
	if(event.currentTarget===event.target){
		onCancel();
	}
}
function onCancel(){
	for(const tbodyTr of tbodyReservation.querySelectorAll('tr')){
		if(tbodyTr.classList.contains('current')){tbodyTr.classList.remove('current');}
	}
	divEditLayer.style.display = 'none';
}
function onTableModified(){
	if(Number(inputTableId.value)===0){
		buttonAdd.disabled = (inputTableName.value==='');
		buttonFix.disabled = true;
		buttonDel.disabled = true;
	}else{
		// fix or del
		buttonAdd.disabled = true;
		buttonFix.disabled = false;
		buttonDel.disabled = false;
	}
}
function onTableUnitChange(event){
		const selected =  event.target[event.target.selectedIndex];
		inputTableUri.style.visibility = (selected.id != 'custom') ? 'hidden' : 'visible';
		onTableModified();
}
async function onExecute(event){
	let tableCustom = 0;
	let tableUri = '';
	if(selectTableUnit[selectTableUnit.selectedIndex].id !='custom'){
		tableCustom = 0;
		tableUri = selectTableUnit[selectTableUnit.selectedIndex].id;
	}else{
		tableCustom = 1;
		tableUri = inputTableUri.value;
	}
	switch(event.target){
		case buttonAdd:{
			try{
				await atsReservationTables.restPost(
					inputTableName.value,
					tableCustom, tableUri,
					inputTableDesc.value,
					inputItemName.value,
					checkboxItemBulk.checked ? 1: 0
				)
			}catch(e){confirm(e.message);}
			break;
		}
		case buttonFix:{
			try{
				await atsReservationTables.restPut(
					Number(inputTableId.value),
					inputTableName.value,
					tableCustom, tableUri,
					inputTableDesc.value,
					inputItemName.value,
					checkboxItemBulk.checked ? 1: 0,
					checkboxDisable.checked ? 1: 0
				)
			}catch(e){confirm(e.message);}
			break;
		}
		case buttonDel:{
			try{
				await atsReservationTables.restDelete(
					Number(inputTableId.value)
				)
			}catch(e){confirm(e.message);}
			break;
		}
		default: confirm('onExecute tagert unknown');
	}
	divEditLayer.style.display = 'none';
	updateReservationList();
}

tbodyReservation.innerHTML = '';
atsColorStyle.loadCss();
updateReservationList();
window.addEventListener( 'resize', onResize);
divEditLayer.addEventListener('click', onOutsideClick);
buttonNew.addEventListener('click', onNewEdit);
buttonUp.addEventListener('click', onOrderExecute);
buttonDown.addEventListener('click', onOrderExecute);
buttonCancel.addEventListener('click', onCancel);
inputTableName.addEventListener('input', onTableModified);
selectTableUnit.addEventListener('change', onTableUnitChange);
inputTableDesc.addEventListener('input', onTableModified);
inputItemName.addEventListener('input', onTableModified);
checkboxItemBulk.addEventListener('change', onTableModified);
checkboxDisable.addEventListener('change', onTableModified);
buttonAdd.addEventListener('click', onExecute);
buttonFix.addEventListener('click', onExecute);
buttonDel.addEventListener('click', onExecute);
onResize();
